=== modified file 'NEWS'
--- NEWS	2008-10-29 23:28:46 +0000
+++ NEWS	2008-10-30 03:56:00 +0000
@@ -239,6 +239,11 @@
     * Trivial documentation fix.
       (John Arbash Meinel, #270471)
 
+    * ``bzr switch`` and ``bzr bind`` will now update the branch nickname if
+      it was previously set. All checkouts will now refer to the bound branch
+      for a nickname if one was not explicitly set.
+      (Marius Kruger, #230903)
+
   DOCUMENTATION:
 
     * Explain revision/range identifiers. (Daniel Clemente)

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2008-10-22 19:07:07 +0000
+++ bzrlib/branch.py	2008-10-30 03:56:00 +0000
@@ -148,8 +148,20 @@
     def get_config(self):
         return BranchConfig(self)
 
-    def _get_nick(self):
-        return self.get_config().get_nickname()
+    def _get_nick(self, possible_transports=None):
+        config = self.get_config()
+        if not config.has_explicit_nickname(): # explicit overrides master
+            try:
+                master = self.get_master_branch(possible_transports)
+                if master is not None:
+                    # return the master branch value
+                    config = master.get_config()
+            except errors.BzrError, e:
+                # Silently fall back to local implicit nick if the master is
+                # unavailable
+                mutter("Could not connect to bound branch, "
+                    "falling back to local nick.\n " + str(e))
+        return config.get_nickname()
 
     def _set_nick(self, nick):
         self.get_config().set_user_option('nickname', nick, warn_masked=True)

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2008-10-17 08:32:52 +0000
+++ bzrlib/builtins.py	2008-10-30 03:56:00 +0000
@@ -3786,6 +3786,8 @@
         except errors.DivergedBranches:
             raise errors.BzrCommandError('These branches have diverged.'
                                          ' Try merging, and then bind again.')
+        if b.get_config().has_explicit_nickname():
+            b.nick = b_other.nick
 
 
 class cmd_unbind(Command):
@@ -4653,7 +4655,8 @@
     of the new location and binds to it.
     
     In both cases, the working tree is updated and uncommitted changes
-    are merged. The user can commit or revert these as they desire.
+    are merged. The user can commit or revert these as they desire. The branch
+    nickname is also updated to be that of the branch being switched to.
 
     Pending merges need to be committed or reverted before using switch.
 
@@ -4672,6 +4675,7 @@
         from bzrlib import switch
         tree_location = '.'
         control_dir = bzrdir.BzrDir.open_containing(tree_location)[0]
+        branch = control_dir.open_branch()
         try:
             to_branch = Branch.open(to_location)
         except errors.NotBranchError:
@@ -4684,6 +4688,9 @@
             to_branch = Branch.open(
                 urlutils.join(this_url, '..', to_location))
         switch.switch(control_dir, to_branch, force)
+        if branch.get_config().has_explicit_nickname():
+            branch = control_dir.open_branch() #get the new branch!
+            branch.nick = to_branch.nick
         note('Switched to branch: %s',
             urlutils.unescape_for_display(to_branch.base, 'utf-8'))
 

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2008-10-01 05:40:45 +0000
+++ bzrlib/commit.py	2008-10-03 23:42:56 +0000
@@ -205,7 +205,8 @@
                config=None,
                message_callback=None,
                recursive='down',
-               exclude=None):
+               exclude=None,
+               possible_master_transports=None):
         """Commit working copy as a new revision.
 
         :param message: the commit message (it or message_callback is required)
@@ -298,7 +299,7 @@
                 raise ConflictsInTree
 
             # Setup the bound branch variables as needed.
-            self._check_bound_branch()
+            self._check_bound_branch(possible_master_transports)
 
             # Check that the working tree is up to date
             old_revno, new_revno = self._check_out_of_date_tree()
@@ -444,7 +445,7 @@
             return
         raise PointlessCommit()
 
-    def _check_bound_branch(self):
+    def _check_bound_branch(self, possible_master_transports=None):
         """Check to see if the local branch is bound.
 
         If it is bound, then most of the commit will actually be
@@ -455,7 +456,8 @@
             raise errors.LocalRequiresBoundBranch()
 
         if not self.local:
-            self.master_branch = self.branch.get_master_branch()
+            self.master_branch = self.branch.get_master_branch(
+                possible_master_transports)
 
         if not self.master_branch:
             # make this branch the reference branch for out of date checks.

=== modified file 'bzrlib/mutabletree.py'
--- bzrlib/mutabletree.py	2008-09-22 05:15:20 +0000
+++ bzrlib/mutabletree.py	2008-10-03 23:42:56 +0000
@@ -181,8 +181,10 @@
         from bzrlib import commit
         if revprops is None:
             revprops = {}
+        possible_master_transports=[]
         if not 'branch-nick' in revprops:
-            revprops['branch-nick'] = self.branch.nick
+            revprops['branch-nick'] = self.branch._get_nick(
+                possible_master_transports)
         author = kwargs.pop('author', None)
         if author is not None:
             if 'author' in revprops:
@@ -194,7 +196,9 @@
         for hook in MutableTree.hooks['start_commit']:
             hook(self)
         committed_id = commit.Commit().commit(working_tree=self,
-            revprops=revprops, *args, **kwargs)
+            revprops=revprops,
+            possible_master_transports=possible_master_transports,
+            *args, **kwargs)
         return committed_id
 
     def _gather_kinds(self, files, kinds):

=== modified file 'bzrlib/tests/blackbox/test_bound_branches.py'
--- bzrlib/tests/blackbox/test_bound_branches.py	2007-12-10 19:20:41 +0000
+++ bzrlib/tests/blackbox/test_bound_branches.py	2008-10-04 00:29:33 +0000
@@ -317,6 +317,33 @@
                             working_dir='tree_1')
         self.assertIs(None, tree.branch.get_bound_location())
 
+    def test_bind_nick(self):
+        """Bind should not update implicit nick."""
+        base = self.make_branch_and_tree('base')
+        child = self.make_branch_and_tree('child')
+        os.chdir('child')
+        self.assertEqual(child.branch.nick, 'child')
+        self.assertEqual(child.branch.get_config().has_explicit_nickname(),
+            False)
+        self.run_bzr('bind ../base')
+        self.assertEqual(child.branch.nick, base.branch.nick)
+        self.assertEqual(child.branch.get_config().has_explicit_nickname(),
+            False)
+
+    def test_bind_explicit_nick(self):
+        """Bind should update explicit nick."""
+        base = self.make_branch_and_tree('base')
+        child = self.make_branch_and_tree('child')
+        os.chdir('child')
+        child.branch.nick = "explicit_nick"
+        self.assertEqual(child.branch.nick, "explicit_nick")
+        self.assertEqual(child.branch.get_config()._get_explicit_nickname(),
+            "explicit_nick")
+        self.run_bzr('bind ../base')
+        self.assertEqual(child.branch.nick, base.branch.nick)
+        self.assertEqual(child.branch.get_config()._get_explicit_nickname(),
+            base.branch.nick)
+
     def test_commit_after_merge(self):
         base_tree, child_tree = self.create_branches()
 

=== modified file 'bzrlib/tests/blackbox/test_nick.py'
--- bzrlib/tests/blackbox/test_nick.py	2007-08-08 01:56:29 +0000
+++ bzrlib/tests/blackbox/test_nick.py	2008-10-04 01:07:42 +0000
@@ -19,6 +19,7 @@
 import os
 
 import bzrlib
+from bzrlib import osutils
 from bzrlib.tests.blackbox import ExternalBase
 
 
@@ -42,3 +43,38 @@
         os.chdir('!repo')
         nick = self.run_bzr('nick')[0]
         self.assertEqual(nick, '!repo\n')
+
+    def test_bound_nick(self):
+        """Check that nick works well for checkouts."""
+        base = self.make_branch_and_tree('base')
+        child = self.make_branch_and_tree('child')
+        os.chdir('child')
+        self.assertEqual(self.run_bzr('nick')[0][:-1], 'child')
+        self.assertEqual(child.branch.get_config().has_explicit_nickname(),
+            False)
+        self.run_bzr('bind ../base')
+        self.assertEqual(self.run_bzr('nick')[0][:-1], base.branch.nick)
+        self.assertEqual(child.branch.get_config().has_explicit_nickname(),
+            False)
+
+        self.run_bzr('unbind')
+        self.run_bzr("nick explicit_nick")
+        self.assertEqual(self.run_bzr('nick')[0][:-1], "explicit_nick")
+        self.assertEqual(child.branch.get_config()._get_explicit_nickname(),
+            "explicit_nick")
+        self.run_bzr('bind ../base')
+        self.assertEqual(self.run_bzr('nick')[0][:-1], base.branch.nick)
+        self.assertEqual(child.branch.get_config()._get_explicit_nickname(),
+            base.branch.nick)
+
+    def test_boundless_nick(self):
+        """Nick defaults to implicit local nick when bound branch is AWOL"""
+        base = self.make_branch_and_tree('base')
+        child = self.make_branch_and_tree('child')
+        os.chdir('child')
+        self.run_bzr('bind ../base')
+        self.assertEqual(self.run_bzr('nick')[0][:-1], base.branch.nick)
+        self.assertEqual(child.branch.get_config().has_explicit_nickname(),
+            False)
+        osutils.rmtree('../base')
+        self.assertEqual(self.run_bzr('nick')[0][:-1], 'child')

=== modified file 'bzrlib/tests/blackbox/test_switch.py'
--- bzrlib/tests/blackbox/test_switch.py	2008-08-07 09:12:47 +0000
+++ bzrlib/tests/blackbox/test_switch.py	2008-10-03 20:57:44 +0000
@@ -20,6 +20,7 @@
 
 import os
 
+from bzrlib.workingtree import WorkingTree
 from bzrlib.tests.blackbox import ExternalBase
 
 
@@ -49,8 +50,58 @@
         self.assertContainsRe(err, 'Switched to branch: .*/branch2.\n')
         self.assertEqual('', out)
 
+    def _test_switch_nick(self, lightweight):
+        """Check that the nick gets switched too."""
+        tree1 = self.make_branch_and_tree('branch1')
+        tree2 = self.make_branch_and_tree('branch2')
+        tree2.pull(tree1.branch)
+        checkout =  tree1.branch.create_checkout('checkout',
+            lightweight=lightweight)
+        self.assertEqual(checkout.branch.nick, tree1.branch.nick)
+        self.assertEqual(checkout.branch.get_config().has_explicit_nickname(),
+            False)
+        self.run_bzr('switch branch2', working_dir='checkout')
+
+        # we need to get the tree again, otherwise we don't get the new branch
+        checkout = WorkingTree.open('checkout')
+        self.assertEqual(checkout.branch.nick, tree2.branch.nick)
+        self.assertEqual(checkout.branch.get_config().has_explicit_nickname(),
+            False)
+
+    def test_switch_nick(self):
+        self._test_switch_nick(lightweight=False)
+
+    def test_switch_nick_lightweight(self):
+        self._test_switch_nick(lightweight=True)
+
+    def _test_switch_explicit_nick(self, lightweight):
+        """Check that the nick gets switched too."""
+        tree1 = self.make_branch_and_tree('branch1')
+        tree2 = self.make_branch_and_tree('branch2')
+        tree2.pull(tree1.branch)
+        checkout =  tree1.branch.create_checkout('checkout',
+            lightweight=lightweight)
+        self.assertEqual(checkout.branch.nick, tree1.branch.nick)
+        checkout.branch.nick = "explicit_nick"
+        self.assertEqual(checkout.branch.nick, "explicit_nick")
+        self.assertEqual(checkout.branch.get_config()._get_explicit_nickname(),
+            "explicit_nick")
+        self.run_bzr('switch branch2', working_dir='checkout')
+
+        # we need to get the tree again, otherwise we don't get the new branch
+        checkout = WorkingTree.open('checkout')
+        self.assertEqual(checkout.branch.nick, tree2.branch.nick)
+        self.assertEqual(checkout.branch.get_config()._get_explicit_nickname(),
+            tree2.branch.nick)
+
+    def test_switch_explicit_nick(self):
+        self._test_switch_explicit_nick(lightweight=False)
+
+    def test_switch_explicit_nick_lightweight(self):
+        self._test_switch_explicit_nick(lightweight=True)
+
     def test_switch_finds_relative_branch(self):
-        """Switch will find 'foo' relative to the branch that the checkout is of."""
+        """Switch will find 'foo' relative to the branch the checkout is of."""
         self.build_tree(['repo/'])
         tree1 = self.make_branch_and_tree('repo/brancha')
         tree1.commit('foo')

